/*
 * Copyright 2010,2013 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 14/06/2013
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 * (http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).
 */
package eu.europa.ec.re3gistry.re3gistryexport.servlet;

import eu.europa.ec.re3gistry.re3gistryexport.manager.Configuration;
import eu.europa.ec.re3gistry.re3gistryexport.xsltrasformations.SolrTransformerMgr;
import java.io.File;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.log4j.Logger;

public class SolrCustomTransformation extends HttpServlet {

    /**
     * Main folder properties
     */
    private static final String KEY_EXPORT_MAINFOLDER_CUSTOM = "export.mainfolder.custom";
    private static final String KEY_EXPORT_MAINFOLDER_NAME = "export.mainfolder.name";
    private static final String KEY_EXPORT_SOLR_FOLDER_NAME = "export.folder.solr";
    private static final String KEY_EXPORT_SOLR_CUSTOM_FOLDER_NAME = "export.folder.solr.custom";
    private static final String KEY_EXPORT_XSL_MAINFOLDER_CUSTOM = "xsl.mainfolder.custom";
    private static final String KEY_EXPORT_XSL_MAINFOLDER = "xsl.mainfolder";
    private static final String KEY_EXPORT_FOLDERIDENTIFIER = "export.context.folderidentifier";
    
    private static final String KEY_EXPORT_FORMAT_AVAILABLE = "transformation.solr.available";
    /**
     * Specific properties values
     */
    private static final String VALUE_TRANSFORMATION_FILEMNAME_BASE_PROPERTY = "transformation.xsl.filename.";
    private static final String VALUE_TRANSFORMATION_FOLDERNAME_BASE_PROPERTY = "transformation.xsl.folderin.";
    private static final String VALUE_TRANSFORMATION_STARTFROM_BASE_PROPERTY = "transformation.xsl.startfrom.";
    private static final String VALUE_TRANSFORMATION_OUTFILEEXTENSION_BASE_PROPERTY = "transformation.xsl.outfileextension.";
    private static final String VALUE_TRANSFORMATION_RECURSIVE_BASE_PROPERTY = "transformation.xsl.recursive.";
    private static final String VALUE_TRANSFORMATION_FOLDERDEPTH_BASE_PROPERTY = "transformation.xsl.folderdepth.";
    /**
     * XML file extension value
     */
    private static final String VALUE_XML_FILE_EXTENSION = "xml";
    /**
     * Common values
     */
    private static final String VALUE_STARTFROM_CUSTOM = "solrcustom";
    private static final String VALUE_STARTFROM_STANDARD = "solr";
    /**
     * Creating logger
     */
    private static Logger log = Logger.getLogger(TransformationServlet.class);

    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        try {
            /**
             * Getting properties
             *
             * Main export folder properties
             */
            String exportMainfolderCustom = Configuration.getInstance().getProperty(KEY_EXPORT_MAINFOLDER_CUSTOM, null);
            String exportMainfolderName = Configuration.getInstance().getProperty(KEY_EXPORT_MAINFOLDER_NAME, "export");
            String exportStandardFolder = Configuration.getInstance().getProperty(KEY_EXPORT_SOLR_FOLDER_NAME, "solr");
            String exportCustomFolder = Configuration.getInstance().getProperty(KEY_EXPORT_SOLR_CUSTOM_FOLDER_NAME, "solrcustom");
            exportMainfolderCustom = (exportMainfolderCustom != null) ? exportMainfolderCustom.trim() : null;
            exportMainfolderName = (exportMainfolderName != null) ? exportMainfolderName.trim() : null;
            exportStandardFolder = (exportStandardFolder != null) ? exportStandardFolder.trim() : null;

            /**
             * XSL Properties
             */
            String xslMainfolderCustom = Configuration.getInstance().getProperty(KEY_EXPORT_XSL_MAINFOLDER_CUSTOM, null);
            String xslMainfolder = Configuration.getInstance().getProperty(KEY_EXPORT_XSL_MAINFOLDER, "xsl.mainfolder");
            xslMainfolderCustom = (xslMainfolderCustom != null) ? xslMainfolderCustom.trim() : null;
            xslMainfolder = (xslMainfolder != null) ? xslMainfolder.trim() : null;

            /**
             * Other formats properties
             */
            String exportAvailableFormats = Configuration.getInstance().getProperty(KEY_EXPORT_FORMAT_AVAILABLE, "");
            exportAvailableFormats = (exportAvailableFormats != null) ? exportAvailableFormats.trim() : null;


            /**
             * Instantiating other common variables
             */
            String contextFolderIdentifier = Configuration.getInstance().getProperty(KEY_EXPORT_FOLDERIDENTIFIER, "WEB-INF");
            contextFolderIdentifier = (contextFolderIdentifier != null) ? contextFolderIdentifier.trim() : null;
            String relativePath = getServletContext().getRealPath(contextFolderIdentifier);
            String folderSeparator = System.getProperty("file.separator");

            /**
             * Initializing the transformation manager
             */
            SolrTransformerMgr t = new SolrTransformerMgr(getServletContext());

            /**
             * Getting base DB export folder
             */
            String folderDbExport = null;
            if (exportMainfolderCustom != null && !exportMainfolderCustom.trim().equals("")) {
                /**
                 * If it is setted the custom export mainfolder, it creates the
                 * folders name (from properties)
                 */
                folderDbExport = exportMainfolderCustom + folderSeparator + exportMainfolderName + folderSeparator + exportStandardFolder;
            } else {
                /**
                 * Else get the servlet context path and then creates the
                 * folders name (from properties)
                 */
                folderDbExport = relativePath.replaceAll(contextFolderIdentifier, "") + exportMainfolderName + folderSeparator + exportStandardFolder;
            }

            //Getting base xsl folder*/
            String folderXsl = null;
            if (xslMainfolderCustom != null && !xslMainfolderCustom.trim().equals("")) {
                /**
                 * If it is setted the custom export mainfolder, it creates the
                 * folders name (from properties)
                 */
                folderXsl = xslMainfolderCustom + folderSeparator + xslMainfolder;
            } else {
                /**
                 * Else get the servlet context path and then creates the
                 * folders name (from properties)
                 */
                folderXsl = relativePath.replaceAll(contextFolderIdentifier, "") + xslMainfolder;
            }

            // ## Other formats transformations processing ## //
            /**
             * Getting custom export folder
             */
            String customExportFolder = null;
            if (exportMainfolderCustom != null && !exportMainfolderCustom.trim().equals("")) {
                /**
                 * If it is setted the custom export mainfolder, it creates the
                 * folders name (from properties)
                 */
                customExportFolder = exportMainfolderCustom + folderSeparator + exportMainfolderName + folderSeparator + exportCustomFolder;
            } else {
                /**
                 * Else get the servlet context path and then creates the
                 * folders name (from properties)
                 */
                customExportFolder = relativePath.replaceAll(contextFolderIdentifier, "") + exportMainfolderName + folderSeparator + exportCustomFolder;
            }

            if (customExportFolder != null && !customExportFolder.trim().equals("") && folderXsl != null && !folderXsl.trim().equals("")) {
                /**
                 * Getting all xml transformation modes
                 */
                String[] mode = exportAvailableFormats.split("-");
                for (int i = 0; i < mode.length; i++) {
                    /**
                     * Getting specific mode properties
                     */
                    String currentXslFilename = Configuration.getInstance().getProperty(VALUE_TRANSFORMATION_FILEMNAME_BASE_PROPERTY + mode[i], null);
                    String currentInSpecificFolder = Configuration.getInstance().getProperty(VALUE_TRANSFORMATION_FOLDERNAME_BASE_PROPERTY + mode[i], null);
                    String startFrom = Configuration.getInstance().getProperty(VALUE_TRANSFORMATION_STARTFROM_BASE_PROPERTY + mode[i], null);
                    String recursiveFlag = Configuration.getInstance().getProperty(VALUE_TRANSFORMATION_RECURSIVE_BASE_PROPERTY + mode[i], null);
                    String folderDepth = Configuration.getInstance().getProperty(VALUE_TRANSFORMATION_FOLDERDEPTH_BASE_PROPERTY + mode[i], null);
                    String outFileExtension = Configuration.getInstance().getProperty(VALUE_TRANSFORMATION_OUTFILEEXTENSION_BASE_PROPERTY + mode[i], null);
                    currentXslFilename = (currentXslFilename != null) ? currentXslFilename.trim() : null;
                    currentInSpecificFolder = (currentInSpecificFolder != null) ? currentInSpecificFolder.trim() : "";
                    recursiveFlag = (recursiveFlag != null) ? recursiveFlag.trim() : null;
                    folderDepth = (folderDepth != null) ? folderDepth.trim() : null;
                    outFileExtension = (outFileExtension != null) ? outFileExtension.trim() : null;

                    /**
                     * Creating complete file and folder names
                     */
                    String xslFile = folderXsl + folderSeparator + currentXslFilename;
                    String inFolder = folderDbExport + folderSeparator + currentInSpecificFolder;
                    try {
                        File tmp = new File(xslFile);
                        
                        if (tmp != null && !tmp.isFile()) {
                            log.info("An error occurred while loading files or folder: cannot load xsl file");
                            continue;
                        }
                        tmp = new File(inFolder);
                        
                        if (tmp != null && !tmp.isDirectory()) {
                            log.info("An error occurred while loading files or folder: cannot ");
                            continue;
                        }
                    } catch (Exception e) {
                        log.info("An error occurred while loading files or folder.", e);
                        continue;
                    }

                    /**
                     * Launching the correct transformation engine
                     */
                    if (recursiveFlag != null && recursiveFlag.trim().equals("false")) {
                        t.transform(new File(inFolder), xslFile, outFileExtension, getServletContext());
                    } else if (recursiveFlag != null && recursiveFlag.trim().equals("true")) {
                        int md = 0;
                        if (folderDepth != null && !folderDepth.trim().equals("")) {
                            try {
                                md = Integer.parseInt(folderDepth);
                            } catch (Exception e) {
                                log.info("An error occurred while reading max depth properties.");
                                continue;
                            }
                        } else {
                            log.info("An error occurred while reading max depth properties: no properties setted");
                            continue;
                        }
                        t.transformRecursively(new File(inFolder), xslFile, outFileExtension, md, getServletContext());
                    } else {
                    }
                }
            } else {
                log.info("An error occurred while reading folders, the task could not be performed.");
            }

            response.sendRedirect("index.jsp");

        } catch (Exception e) {
            log.error("An error has occurred while executing the transformation job.", e);
        }
    }
}
